export function delegate(selector, handler) {
    return function (event) {
        var node, path = event.composedPath();
        while ((node = path.shift()) && node !== event.currentTarget)
            if (node instanceof HTMLElement && node.matches(selector))
                return handler.call(this, event, node, event.detail);
    };
}
export var documentReady = new Promise(function (resolve) {
    document.addEventListener('DOMContentLoaded', resolve);
    self.addEventListener('load', resolve);
    setTimeout(function check() {
        document.readyState === 'complete' ? resolve() : setTimeout(check);
    });
});
export function promisify(scope, element) {
    return new Promise(function (resolve, reject) {
        function end(event) {
            resolve(event), clean();
        }
        function cancel(event) {
            reject(event), clean();
        }
        function clean() {
            element.removeEventListener(scope + 'end', end);
            element.removeEventListener(scope + 'cancel', cancel);
        }
        element.addEventListener(scope + 'end', end);
        element.addEventListener(scope + 'cancel', cancel);
    });
}
//# sourceMappingURL=event.js.map